#define _CRT_SECURE_NO_WARNINGS

#include <dxapi/dxapi.h>
#include <chrono>
#include <stdio.h>
#include <conio.h>
#include <memory>

using namespace std;
using namespace DxApi;

#define HOST "dxtick://localhost:8022"
#define STREAM "tick.stream"
#define NUM_MSG 600


// Get nanosecond UTC time using std::chrono
static int64_t now_ns()
{
    return chrono::duration_cast<chrono::nanoseconds>(chrono::system_clock::now().time_since_epoch()).count();
}


static void createSampleStream(TickDb *db)
{
    // TODO: NOT SUPPORTED. We can create a stream with given XML schema in text form, but nothing else.
}

enum class AggressorSide {
    BUY,
    SELL
};

//enum class MarketEventType {
//    Bid,
//    Offer,
//    Trade,
//    IndexValue,
//    OpeningProce,
//    ClosingProce,
//};

//class InstrumentMessage {
//public:
//    InstrumentIdentity  instrument;
//    int64_t             timestamp;
//};
//
//
//class MarketMessage : public InstrumentMessage {
//public:
//    Nullable<int64_t>   sequenceNumber;
//};


class TradeMessage : public MarketMessage {
public:
    string              exchangeCode;
    double              price;
    double              size;
    string              condition;
    int                 aggressorSide;
    bool                beginMatch;
    double              netPriceChange;
    int                 eventType;

    bool has_exchangeCode, has_price, has_size, has_condition, has_agressorSide,
         has_beginMatch, has_netPriceChange, has_eventType;


    inline void clear()
    {
        has_sequenceNumber = has_exchangeCode = has_price = has_size = has_condition = has_agressorSide
            = has_beginMatch = has_netPriceChange = has_eventType = false;
    }


    inline void decode(InstrumentMessage &m, DataReader &r)
    {
        *(static_cast<InstrumentMessage *>(this)) = m;
        has_sequenceNumber = r.readInt64(sequenceNumber);
        has_exchangeCode = r.readAlphanumeric(exchangeCode, 10);
        has_price = r.readDecimal(price);
        has_size = r.readDecimal(size);
        has_condition = r.readUTF8(condition);
        has_agressorSide = r.readEnum8(aggressorSide);
        has_beginMatch = r.readNullableBoolean(beginMatch);
        has_netPriceChange = r.readDecimal(netPriceChange);
        has_eventType = r.readEnum8(eventType);
    }


    TradeMessage()
    {
        clear();
    }
};


class BestBidOfferMessage : public MarketMessage {
public:
    bool                isNational;
    double              bidPrice;
    double              bidSize;
    string              bidExchange;
    double              offerPrice;
    double              offerSize;
    string              offerExchange;

    bool has_isNational, has_bidPrice, has_bidSize, has_bidExchange, has_offerPrice, has_offerSize, has_offerExchange;


    inline void clear()
    {
        has_sequenceNumber = has_isNational
            = has_bidPrice = has_bidSize = has_bidExchange
            = has_offerPrice = has_offerSize = has_offerExchange = false;
    }


    inline void decode(InstrumentMessage &m, DataReader &r)
    {
        *(static_cast<InstrumentMessage *>(this)) = m;
        has_sequenceNumber = r.readInt64(sequenceNumber);
        has_bidPrice = r.readDecimal(bidPrice);
        has_bidSize = r.readDecimal(bidSize);
        has_bidExchange = r.readAlphanumeric(bidExchange, 10);
        has_offerPrice = r.readDecimal(offerPrice);
        has_offerSize = r.readDecimal(offerSize);
        has_offerExchange = r.readAlphanumeric(offerExchange, 10);
    }


    BestBidOfferMessage()
    {
        clear();
    }
};


static void loadData(TickDb *db)
{
    auto stream = db->getStream(STREAM);
    stream->clear();
    LoadingOptions opt;
    intptr_t nMessagesWritten;
    unique_ptr<TickLoader> loader(stream->createLoader(opt));
    DataWriter &writer = loader->getWriter();

    // The current C++ API is very low-level, it lacks schema interpreter.
    // There are several version of beginMessage() and codec methods, with different convenience/performance tradeoffs
    // They were changed for greater convenience for C# wrapper, currently less than optimal for direct use

    enum MessageTypes {
        BBO,
        TRADE
    };

    loader->registerMessageType(TRADE, "deltix.qsrv.hf.pub.TradeMessage");
    loader->registerMessageType(BBO, "deltix.qsrv.hf.pub.BestBidOfferMessage");

    auto ibmInstr = loader->getInstrumentId(InstrumentIdentity(DxApi::InstrumentType::EQUITY, "IBM"));

    int64_t time = now_ns();

    for (nMessagesWritten = 0; nMessagesWritten < NUM_MSG; ++nMessagesWritten) {
        auto ii = nMessagesWritten;
        double simulatedPrice = 126 + ii / 200.0;
        double simulatedSize = (ii % 10) * 100.0;
        int64_t simulatedTimestamp = time + ii * 100000000;

        switch (nMessagesWritten & 3) {
        case 0:
            //  Generate a trade tick
            loader->beginMessage(TRADE, ibmInstr, simulatedTimestamp);
            writer.writeInt64(INT64_NULL);                      // Sequence number
            writer.writeAlphanumericNull(10);                   // Exchange
            writer.writeDecimal(simulatedPrice, 2U);            // Price
            writer.writeDecimal(simulatedSize, 2U);             // Size
            break;

        case 1:
            //  Generate a two-sided quote
            loader->beginMessage(BBO, ibmInstr, simulatedTimestamp);
            writer.writeInt64(INT64_NULL);                      // Sequence number

            writer.writeDecimal(simulatedPrice - 0.01, 2U);     // Bidprice
            writer.writeDecimal(simulatedSize * 5, 2U);         // Bidsize
            writer.writeAlphanumericNull(10);                   // BidExchange
            writer.writeDecimal(simulatedPrice + 0.01);         // offerPrice
            writer.writeDecimal(simulatedSize * 4);             // offerSize;

            break;

        case 2:
            //  Generate an one-sided bid quote
            loader->beginMessage(BBO, ibmInstr, simulatedTimestamp);
            writer.writeInt64(INT64_NULL);                      // Sequence number
            
            writer.writeDecimal(simulatedPrice - 0.01, 2U);     // BidPrice
            writer.writeDecimal(simulatedSize * 5, 2U);         // BidSize
            //writer.writeAlphanumericNull(10);                 // BidExchange
            //writer.writeDecimal(FLOAT64_NULL);                // OfferPrice
            //writer.writeDecimal(FLOAT64_NULL);                // OfferSize
            break;

        case 3:
            //  Generate an one-sided offer quote
            loader->beginMessage(BBO, ibmInstr, simulatedTimestamp);
            writer.writeInt64(INT64_NULL);                      // Sequence number
            
            writer.writeDecimal(FLOAT64_NULL);                  // BidPrice
            writer.writeDecimal(FLOAT64_NULL);                  // BidSize
            writer.writeAlphanumericNull(10);                   // BidExchange
            writer.writeDecimal(simulatedPrice - 0.01, 2U);     // OfferPrice
            writer.writeDecimal(simulatedSize * 5, 2U);         // OfferSize

            break;
        }

        loader->send();
    }


    loader->close();
    puts("Loading done");
}


static bool contains(const std::string &str, const char * substr)
{
    return string::npos != str.find(substr);
}


static void readData(TickDb *db)
{
    auto stream = db->getStream(STREAM);
    SelectionOptions opt;
    unique_ptr<TickCursor> cursor(stream->select(TIMESTAMP_NULL, opt, NULL, NULL));

    DataReader &reader = cursor->getReader();
    MarketMessage msg;
    TradeMessage trade;
    BestBidOfferMessage bbo;

    msg.cursor = cursor.get();

    enum MessageTypes {
        BBO,
        TRADE,
        MESSAGE_TYPES_COUNT
    };

    string nullStr("<NULL>");

    unsigned knownMessages[0x100];
    memset(knownMessages, -1, sizeof(knownMessages));

    // Polymorphic message reading is cumbersome in this version, because registerMessageTypes stopped being supported, as it was not necessary for C# layer.
    while (cursor->next(&msg)) {
        printf("%s\t", msg.toString().c_str());
    again:
        switch (knownMessages[msg.typeId]) {
        case BBO:
            {
                bbo.decode(msg, reader);
                std::stringstream ss;
                ss << "bidPrice: ";
                bbo.has_bidPrice ? ss << bbo.bidPrice : ss << nullStr;
                ss << "; bidSize: ";
                bbo.has_bidSize ? ss << bbo.bidSize : ss << nullStr;
                ss << "; offerSize: ";
                bbo.has_offerPrice ? ss << bbo.offerPrice : ss << nullStr;
                ss << "; offerSize: ";
                bbo.has_offerSize ? ss << bbo.offerSize : ss << nullStr;
                puts(ss.str().c_str());
            }

            break;

        case TRADE:
            {
                trade.decode(msg, reader);
                //printf("price: %s; size: %s\n", trade.has_price ? toString(trade.price).c_str(), trade.price);

                std::stringstream ss;
                ss << "price: ";
                trade.has_price ? ss << trade.price : ss << nullStr;
                ss << "; size: ";
                trade.has_size ? ss << trade.size : ss << nullStr;
                puts(ss.str().c_str());
            }
            break;

        default:
            const string * typeName = cursor->getMessageTypeName(msg.typeId);
            assert(NULL != typeName);
            if (contains(*typeName, "TradeMessage")) {
                knownMessages[msg.typeId] = TRADE;
            }
            else if (contains(*typeName, "BestBidOfferMessage")) {
                knownMessages[msg.typeId] = BBO;
            }
            else {
                puts("Unsupported message!");
                exit(1);
            }

            goto again;
        }
    }

    
    cursor->close();
    puts("Reading done");
}


int main(int argc, char* argv[])
{
    // We assume that the stream is already created. Code is written with assumption that a specific schema is used
    // There is no API similar to StreamConfigurationHelper available via C++ API,
    
    // For quite a while it was developed with the sole purpose of being wrapped with C++/CLI & C#
    // so it is not very convenient to use at the moment

    try {
        // This code sequnce ensures that destructors are called automatically
        unique_ptr<TickDb> db(TickDb::createFromUrl(argc > 1 ? argv[1] : HOST));

        db->open(false);
        //createSampleStream(db); // Should be already created
        loadData(db.get());
        readData(db.get());
    }
    catch (exception &e) {
        puts(e.what());
    }
    catch (...) {
        puts("System exception");
    }

    return 0;
}