#ifndef DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_BASEENTRY_INC
#define DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_BASEENTRY_INC

#include <string>
#include <stdint.h>

#include "dxapi/dxapi.h"
#include "dfp/DecimalNative.hpp"
#include "src/codecs/NativeMessage.h"

using namespace std;
using namespace DxApi;

using namespace deltix::dfp;

namespace dx_BITTREX::deltix::timebase::api::messages::universal {

class Dx_BaseEntry : public NativeMessage {
private:
    std::string __class_type__ = "deltix.timebase.api.messages.universal.BaseEntry";
    std::string __class_guid__ = "64406423fb2b4ajpjsoe_15";

public:
    virtual std::string & getTypeName() { return __class_type__; }
    virtual std::string & getGuid() { return __class_guid__; }

    virtual void decode(DataReader &reader) {
        has_contractId_ = reader.readAlphanumeric(contractId_, 10);
        has_exchangeId_ = reader.readAlphanumeric(exchangeId_, 10);
        isImplied_ = reader.readNullableBooleanInt8();
    }

    virtual void encode(DataWriter &writer) {
        has_contractId_ ? writer.writeAlphanumeric(10, contractId_) :  writer.writeAlphanumericNull(10);
        has_exchangeId_ ? writer.writeAlphanumeric(10, exchangeId_) :  writer.writeAlphanumericNull(10);
        writer.writeNullableBoolean(isImplied_, isImplied_ == Constants::BOOL_NULL);
    }

    inline void clear() {
        contractId_.clear();
        has_contractId_ = false;
        exchangeId_.clear();
        has_exchangeId_ = false;
        isImplied_ = Constants::BOOL_NULL;
    }

    inline void setContractId(const string &value) {
        this->contractId_ = value;
        has_contractId_ = true;
    }

    inline void setExchangeId(const string &value) {
        this->exchangeId_ = value;
        has_exchangeId_ = true;
    }

    inline void setIsImplied(uint8_t value) {
        this->isImplied_ = value;
    }

    virtual string toString() {
        stringstream ss;
        ss << NativeMessage::toString() << ", ";
        ss << "  contractId_: "; 
        if (!has_contractId_) { ss << "[NULL]"; } else { ss << contractId_; }
        ss << ", ";
        ss << "  exchangeId_: "; 
        if (!has_exchangeId_) { ss << "[NULL]"; } else { ss << exchangeId_; }
        ss << ", ";
        ss << "  isImplied_: "; 
        if (isImplied_ == Constants::BOOL_NULL) { ss << "[NULL]"; } else { ss << (int) isImplied_; }
        return ss.str();
    };

    string contractId_;
    string exchangeId_;
    uint8_t isImplied_;

    bool has_contractId_ = false;
    bool has_exchangeId_ = false;
};

} // namespace dx_BITTREX::deltix::timebase::api::messages::universal

#endif /* DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES__UNIVERSAL_DX_BASEENTRY_INC */
